/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "physics.h"

collisionresult_t physicsCheckCircleCircle(
  fixed248_t circle0x, fixed248_t circle0y, fixed248_t circle0r,
  fixed248_t circle1x, fixed248_t circle1y, fixed248_t circle1r
) {
  collisionresult_t result;
  // Compute vector between centers
  fixed248_t dx = fx248Subfx248(circle1x, circle0x);
  fixed248_t dy = fx248Subfx248(circle1y, circle0y);

  // Distance squared between centers
  fixed248_t distSq = fx248Addfx248(
    fx248Mulfx248(dx, dx),
    fx248Mulfx248(dy, dy)
  );

  // Sum of radii
  fixed248_t rSum = fx248Addfx248(circle0r, circle1r);
  fixed248_t rSumSq = fx248Mulfx248(rSum, rSum);

  if(distSq > rSumSq) {
    // No collision
    result.hit = false;
    return result;
  }

  // Collision: calculate normal and penetration depth
  fixed248_t dist = fx248Sqrt(distSq);

  // If centers are the same, pick arbitrary normal (1,0)
  if(dist == 0) {
    result.normalX = FIXED248(1,0);
    result.normalY = 0;
    result.depth = rSum;
  } else {
    // Normalized direction from circle0 to circle1
    result.normalX = fx248Divfx248(dx, dist);
    result.normalY = fx248Divfx248(dy, dist);
    // Penetration depth = sum of radii - distance
    result.depth = fx248Subfx248(rSum, dist);
  }
  result.hit = true;
  return result;
}

collisionresult_t physicsCheckCircleAABB(
  fixed248_t circleX, fixed248_t circleY, fixed248_t circleR,
  fixed248_t aabbX, fixed248_t aabbY,
  fixed248_t aabbWidth, fixed248_t aabbHeight
) {
  collisionresult_t result;

  // Find the closest point on the AABB to the circle center
  fixed248_t closestX = fx248Max(
    aabbX, fx248Min(circleX, fx248Addfx248(aabbX, aabbWidth))
  );
  fixed248_t closestY = fx248Max(
    aabbY, fx248Min(circleY, fx248Addfx248(aabbY, aabbHeight))
  );

  // Vector from circle center to closest point
  fixed248_t dx = fx248Subfx248(closestX, circleX);
  fixed248_t dy = fx248Subfx248(closestY, circleY);

  // Distance squared from circle center to closest point
  fixed248_t distSq = fx248Addfx248(fx248Mulfx248(dx, dx), fx248Mulfx248(dy, dy));

  // Check if distance is less than radius squared
  if(distSq > fx248Mulfx248(circleR, circleR)) {
    result.hit = false;
    return result;
  }

  // Collision: calculate normal and penetration depth
  fixed248_t dist = fx248Sqrt(distSq);
  
  if(dist <= FIXED248(0, 1)) {
    // Circle center is at the AABB corner
    result.normalX = FIXED248(1,0);
    result.normalY = FIXED248(0,1);
    result.depth = circleR;
  } else {
    // Normalized direction from circle center to closest point
    result.normalX = fx248Divfx248(dx, dist);
    result.normalY = fx248Divfx248(dy, dist);
    // Penetration depth = radius - distance
    result.depth = fx248Subfx248(circleR, dist);
  }
  
  result.hit = true;
  return result;
}